import matplotlib.pyplot as plt


def plot_scenarios():

    scenarios = ['1', '2', '3']
    curr = [0, 0, 0]
    model_36 = [-0.112, 0.088, 0.640]
    model_37 = [-0.085, 0.091, 0.631]
    model_30 = [-0.445, 0.039, 0.679]

    fig, ax = plt.subplots()

    ax.plot(scenarios, curr)
    ax.plot(scenarios, model_36)
    ax.plot(scenarios, model_37)
    ax.plot(scenarios, model_30)
    plt.ylabel("Utility")
    plt.xlabel("Scenario")

    ax.set_xticks(scenarios)

    plt.grid()
    plt.legend(('Current', 'Model 36', 'Model 37', 'Model 30'),
               loc='lower right')
    plt.savefig("scenarios.png")
    plt.show()


def plot_kbest_avg():
    sizes = [1000, 13500, 15500, 17500, 25000]
    f1 = [0.455, 0.463, 0.469, 0.460, 0.451]
    precision = [0.646, 0.660, 0.690, 0.648, 0.639]
    recall = [0.351, 0.356, 0.356, 0.356, 0.348]

    fig, ax = plt.subplots()

    ax.plot(sizes, f1)
    ax.plot(sizes, precision)
    ax.plot(sizes, recall)
    plt.ylabel("Score")
    plt.xlabel("K best features")

    ax.set_xticks(sizes)

    plt.grid()
    plt.legend(('F1', 'Precision', 'Recall'),
               loc='best')
    plt.savefig("kbest_avg")
    plt.show()


def plot_trees_avg():
    sizes = [1000, 5000, 7500, 10000, 15000]
    f1 = [0.479, 0.481, 0.474, 0.484, 0.481]
    precision = [0.722, 0.733, 0.729, 0.735, 0.740]
    recall = [0.358, 0.358, 0.351, 0.361, 0.356]
    fig, ax = plt.subplots()

    ax.plot(sizes, f1)
    ax.plot(sizes, precision)
    ax.plot(sizes, recall)
    ax.set_xticks(sizes)
    plt.ylabel("Score")
    plt.xlabel("Number of trees")
    plt.grid()
    plt.legend(('F1', 'Precision', 'Recall'),
               loc='best')
    plt.savefig("trees_avg")
    plt.show()


def plot_fraction_avg():
    sizes = [226, 453, 906, 1811]
    f1 = [0.300, 0.331, 0.416, 0.484]
    precision = [0.545, 0.633, 0.660, 0.735]
    recall = [0.207, 0.224, 0.304, 0.361]
    fig, ax = plt.subplots()

    ax.plot(sizes, f1)
    ax.plot(sizes, precision)
    ax.plot(sizes, recall)
    ax.set_xticks(sizes)
    plt.ylabel("Score")
    plt.xlabel("Number of samples")
    plt.grid()
    plt.legend(('F1', 'Precision', 'Recall'),
               loc='lower right')
    plt.savefig("fraction_avg")
    plt.show()


def plot_sizes_avg():
    sizes = ["(180, 50)", "(360, 101)", "(719, 202)", "(1437, 404)"]
    f1 = [0.448, 0.410, 0.408, 0.402]
    precision = [0.516, 0.502, 0.498, 0.485]
    recall = [0.396, 0.346, 0.346, 0.343]
    fig, ax = plt.subplots()

    ax.plot(sizes, f1)
    ax.plot(sizes, precision)
    ax.plot(sizes, recall)
    ax.set_xticks(sizes)
    plt.ylabel("Score")
    plt.xlabel("Resolution")
    plt.grid()
    plt.legend(('F1', 'Precision', 'Recall'),
               loc='best')

    plt.savefig("sizes_avg.png")
    plt.show()


def plot_sensors_avg():
    sizes = ['Full', 'Standard', 'Squats', 'VSH']
    f1 = [0.448, 0.438, 0.331, 0.335]
    precision = [0.516, 0.489, 0.520, 0.593]
    recall = [0.396, 0.396, 0.243, 0.233]
    fig, ax = plt.subplots()

    ax.plot(sizes, f1)
    ax.plot(sizes, precision)
    ax.plot(sizes, recall)
    ax.set_xticks(sizes)
    plt.ylabel("Score")
    plt.xlabel("Sensor subset")
    plt.grid()
    plt.legend(('F1', 'Precision', 'Recall'),
               loc='center right')
    plt.savefig("sensors_avg.png")
    plt.show()


def plot_top_avg():
    sizes = ['36', '37', '30']
    f1 = [0.498, 0.495, 0.494]
    precision = [0.735, 0.754, 0.569]
    recall = [0.376, 0.368, 0.436]
    fig, ax = plt.subplots()

    ax.plot(sizes, f1)
    ax.plot(sizes, precision)
    ax.plot(sizes, recall)
    ax.set_xticks(sizes)
    plt.ylabel("Score")
    plt.xlabel("Model")
    plt.grid()
    plt.legend(('F1', 'Precision', 'Recall'),
               loc='best')
    plt.savefig("top_avg.png")
    plt.show()


plot_scenarios()
plot_kbest_avg()
plot_trees_avg()
plot_fraction_avg()
plot_sizes_avg()
plot_sensors_avg()
plot_top_avg()
