from PIL import Image
import os.path
import matplotlib.pyplot as plt
import numpy as np
import csv

values = []

colours = [
    [128, 128, 128],
    [255, 255, 83],
    [253, 151, 249],
    [255, 0, 128],
    [1, 255, 255],
    [255, 83, 255],
    [67, 106, 227],
    [255, 141, 0],
    [146, 113, 218],
    [56, 255, 103],
    [139, 1, 140],
    [238, 128, 131],
    [1, 153, 53]
]

standard_subset_colours = colours[:7]
squat_subset_colours = colours[7:10]
vsh_subset_colours = colours[10:]


def get_rail_side_array(rail_side_filename):
    rail_side_array = []
    with open(rail_side_filename, 'r') as csvfile:
        reader = csv.reader(csvfile, delimiter=';')
        for row in reader:
            rail_side_array.append(row)
    return rail_side_array


def extract_Bscan(filename, img, rail_side_filename):
    # extract upper graph if the rail on the right side is the one with defect, otherwise the left
    found_rail_side = False
    rail_side_array = get_rail_side_array(rail_side_filename=rail_side_filename)
    for row in rail_side_array:
        if filename.replace('.jpg', '') == row[0]:
            if row[1] == "Høyre":
                # Original upper graph region is (49, 159, 1486, 625)
                bscan = img.crop((49, 159, 1486, 159 + 404))  # max y found to be 404
                found_rail_side = True
            elif row[1] == "Venstre":
                # Original lower graph region is (49, 663, 1486, 1129)
                bscan = img.crop((49, 663, 1486, 663 + 404))  # max y found to be 404
                found_rail_side = True

    if found_rail_side == False:
        raise Exception("No rail side was found for the id" + str(filename));

    return bscan


def remove_grid(bscan, extract_sensor_subset):
    # Removing grid and converting to black and white")
    pixels = bscan.load()
    for i in range(bscan.size[0]):
        for j in range(bscan.size[1]):
            if pixels[i, j][0] > 200 and pixels[i, j][1] > 200 and pixels[i, j][2] > 200:
                pixels[i, j] = (255, 255, 255)
            if not extract_sensor_subset:
                if pixels[i, j] != (255, 255, 255):
                    pixels[i, j] = (0, 0, 0)
    return bscan


def extract_sensor_subset(threshold_value, bscan):
    t = threshold_value
    pixels = bscan.load()
    for i in range(bscan.size[0]):
        for j in range(bscan.size[1]):
            colour_changed = False
            for colour in colours[10:]:
                if pixels[i, j][0] in range(colour[0] - t, colour[0] + t) \
                        and pixels[i, j][1] in range(colour[1] - t, colour[1] + t) and \
                        pixels[i, j][2] in range(colour[2] - t, colour[2] + t):
                    pixels[i, j] = (0, 0, 0)
                    colour_changed = True
            if not colour_changed:
                pixels[i, j] = (255, 255, 255)
    return bscan


def resize(bscan, resolution):
    return bscan.resize(resolution, Image.NEAREST)


def convert_to_vector(bscan, resolution):
    vector = np.zeros(resolution, dtype=int)
    pixels = bscan.load()
    for i in range(len(vector)):
        for j in range(len(vector[0])):
            if pixels[i, j] == (0, 0, 0):
                vector[i][j] = 1
    return vector


resolution = (1437, 404)
use_sensor_subset = False
subset_colours = standard_subset_colours #standard_subset_colours, squat_subset_colours or vsh_subset_colours
save_bscan_image = True
visualize_bscan = True
threshold_value = 50
rail_side_filename = "rail_side.csv"

#retrieve filenames in current folder
filenames = os.listdir()

for filename in filenames:
    if filename.endswith('.jpg'):

        print("Opening file " + filename)
        img = Image.open(filename)
        r, g, b = img.getpixel((1,1))

        bscan = extract_Bscan(filename=filename, img=img, rail_side_filename=rail_side_filename)
        bscan = remove_grid(bscan=bscan, extract_sensor_subset=extract_sensor_subset)

        bscan = resize(bscan=bscan, resolution=resolution)

        if use_sensor_subset:
            bscan = extract_sensor_subset(threshold_value=threshold_value, bscan=bscan)

        if save_bscan_image:
            bscan.save('bscans/' + filename)

        if visualize_bscan:
            plt.imshow(bscan)
            plt.show()

        vector = convert_to_vector(bscan=bscan, resolution=resolution)
        values.append(vector)

np.save('values/all_bscans', values)




